Package: SparseSignatures
Version: 2.20.0
Date: 2025-09-23
Title: SparseSignatures
Authors@R: c(
    person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")),
    person("Avantika", "Lal", role=c("aut")),
    person("Keli", "Liu", role=c("ctb")),
    person("Luca", "De Sano", role=c("cre","aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")),
    person("Robert", "Tibshirani", role=c("ctb")),
    person("Arend", "Sidow", role=c("aut")))
Depends: R (>= 4.1.0), NMF
Imports: nnlasso, nnls, parallel, data.table, Biostrings,
        GenomicRanges, IRanges, BSgenome, GenomeInfoDb, ggplot2,
        gridExtra, reshape2, RhpcBLASctl
Suggests: BiocGenerics, BSgenome.Hsapiens.1000genomes.hs37d5,
        BiocStyle, testthat, knitr,
Name: An R package for the extraction of sparse mutational signatures
        from whole genome sequencing data
Description: 
    Point mutations occurring in a genome can be divided into 96 categories based on the base being mutated, the base it is mutated into and its two flanking bases. Therefore, for any patient, it is possible to represent all the point mutations occurring in that patient's tumor as a vector of length 96, where each element represents the count of mutations for a given category in the patient. A mutational signature represents the pattern of mutations produced by a mutagen or mutagenic process inside the cell. Each signature can also be represented by a vector of length 96, where each element represents the probability that this particular mutagenic process generates a mutation of the 96 above mentioned categories. In this R package, we provide a set of functions to extract and visualize the mutational signatures that best explain the mutation counts of a large number of patients.
Encoding: UTF-8
License: file LICENSE
URL: https://github.com/danro9685/SparseSignatures
BugReports: https://github.com/danro9685/SparseSignatures
biocViews: BiomedicalInformatics, SomaticMutation
RoxygenNote: 7.3.2
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/SparseSignatures
git_branch: RELEASE_3_22
git_last_commit: 9b8de11
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:42:14 UTC; biocbuild
Author: Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>),
  Avantika Lal [aut],
  Keli Liu [ctb],
  Luca De Sano [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-9618-3774>),
  Robert Tibshirani [ctb],
  Arend Sidow [aut]
Maintainer: Luca De Sano <luca.desano@gmail.com>
