# Generated by roxygen2: do not edit by hand

export(import.trinucleotides.counts)
export(lambdaRangeAlphaEvaluation)
export(lambdaRangeBetaEvaluation)
export(nmfLasso)
export(nmfLassoBootstrap)
export(nmfLassoCV)
export(patients.plot)
export(sigAssignmentCV)
export(sigAssignmentEvaluation)
export(sigAssignmentLasso)
export(signatures.plot)
export(startingBetaEstimation)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(NMF)
import(RhpcBLASctl)
import(ggplot2)
import(gridExtra)
import(nnlasso)
import(nnls)
import(parallel)
importFrom(BSgenome,getSeq)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,complement)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,subseq)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(reshape2,melt)
